@@ -575,6 +575,79 @@ CC_clear_col_info(ConnectionClass *self, BOOL destroy)
 	}
 }
 
+static void
+CC_set_locale_encoding(ConnectionClass *self, const char * encoding)
+{
+	char	*currenc = self->locale_encoding;
+
+	if (encoding)
+		self->locale_encoding = strdup(encoding);
+	else
+		self->locale_encoding = NULL;
+	if (currenc)
+		free(currenc);
+}
+
+static void
+CC_determine_locale_encoding(ConnectionClass *self)
+{
+	const char *dbencoding = PQparameterStatus(self->pqconn, "client_encoding");
+	const char *encoding;
+
+	if (self->locale_encoding) /* already set */
+                return;
+	encoding = derive_locale_encoding(dbencoding);
+	if (!encoding)
+		encoding = "SQL_ASCII";
+	CC_set_locale_encoding(self, encoding);
+}
+
+static void
+CC_set_client_encoding(ConnectionClass *self, const char * encoding)
+{
+	char	*currenc = self->original_client_encoding;
+
+	if (encoding)
+	{
+		self->original_client_encoding = strdup(encoding);
+		self->ccsc = pg_CS_code(encoding);
+	}
+	else
+	{
+		self->original_client_encoding = NULL;
+		self->ccsc = SQL_ASCII;
+	}
+	self->mb_maxbyte_per_char = pg_mb_maxlen(self->ccsc);
+	if (currenc)
+		free(currenc);
+}
+
+int
+CC_send_client_encoding(ConnectionClass *self, const char * encoding)
+{
+	const char *dbencoding = PQparameterStatus(self->pqconn, "client_encoding");
+
+	if (encoding && (!dbencoding || stricmp(encoding, dbencoding)))
+	{
+                char	query[64];
+		QResultClass	*res;
+		int	errnum = CC_get_errornumber(self);
+		BOOL	cmd_success;
+
+		snprintf(query, sizeof(query), "set client_encoding to '%s'", encoding);
+		res = CC_send_query(self, query, NULL, IGNORE_ABORT_ON_CONN | ROLLBACK_ON_ERROR, NULL);
+		cmd_success = QR_command_maybe_successful(res);
+		QR_Destructor(res);
+		CC_set_errornumber(self, errnum);
+
+		if (!cmd_success)
+			return SQL_ERROR;
+	}
+	CC_set_client_encoding(self, encoding);
+
+	return SQL_SUCCESS;
+}
+
 /* This is called by SQLDisconnect also */
 char
 CC_cleanup(ConnectionClass *self, BOOL keepCommunication)
@@ -649,6 +722,11 @@ CC_cleanup(ConnectionClass *self, BOOL keepCommunication)
 			free(self->original_client_encoding);
 			self->original_client_encoding = NULL;
 		}
+		if (self->locale_encoding)
+		{
+			free(self->locale_encoding);
+			self->locale_encoding = NULL;
+		}
 		if (self->server_encoding)
 		{
 			free(self->server_encoding);
@@ -904,21 +982,17 @@ static char CC_initial_log(ConnectionClass *self, const char *func)
 		 ci->drivers.bools_as_char,
 		 TABLE_NAME_STORAGE_LEN);
 
-	encoding = check_client_encoding(ci->conn_settings);
-	if (encoding)
-		self->original_client_encoding = encoding;
-	else
+	if (NULL == self->locale_encoding)
 	{
-		encoding = check_client_encoding(ci->drivers.conn_settings);
-		if (encoding)
-			self->original_client_encoding = encoding;
+		encoding = check_client_encoding(ci->conn_settings);
+		if (!encoding)
+			encoding = check_client_encoding(ci->drivers.conn_settings);
+		CC_set_locale_encoding(self, encoding);
+		qlog("                extra_systable_prefixes='%s', conn_settings='%s' conn_encoding='%s'\n",
+			ci->drivers.extra_systable_prefixes,
+			PRINT_NAME(ci->drivers.conn_settings),
+			encoding ? encoding : "");
 	}
-	if (self->original_client_encoding)
-		self->ccsc = pg_CS_code(self->original_client_encoding);
-	qlog("                extra_systable_prefixes='%s', conn_settings='%s' conn_encoding='%s'\n",
-		 ci->drivers.extra_systable_prefixes,
-		 PRINT_NAME(ci->drivers.conn_settings),
-		 encoding ? encoding : "");
 	if (self->status == CONN_DOWN)
 	{
 		CC_set_error_if_not_set(self, CONN_OPENDB_ERROR, "Connection broken.", func);
@@ -1007,25 +1081,32 @@ CC_connect(ConnectionClass *self, char *salt_para)
 	CC_lookup_lo(self);			/* a hack to get the oid of
 						   our large object oid type */
 
-	/* Multibyte handling */
-	CC_lookup_characterset(self);
-	if (CC_get_errornumber(self) > 0)
-	{
-		ret = 0;
-		goto cleanup;
-	}
+	/*
+	 *		Multibyte handling
+	 *
+	 *	Send 'UTF8' when required Unicode behavior, otherwise send
+	 *	locale encodings.
+	 */
+	CC_determine_locale_encoding(self); /* determine the locale_encoding */
 #ifdef UNICODE_SUPPORT
 	if (CC_is_in_unicode_driver(self))
 	{
-		const char *curr = PQparameterStatus(self->pqconn, "client_encoding");
-		if (!curr || strcmp(curr, "UTF8") != 0)
+		if (!SQL_SUCCEEDED(CC_send_client_encoding(self, "UTF8")))
 		{
-			QResultClass	*res;
-			res = CC_send_query(self, "set client_encoding to 'UTF8'", NULL, 0, NULL);
-			QR_Destructor(res);
+			CC_set_error(self, CONN_OPENDB_ERROR, "Failed to set client_encoding to UTF8", __FUNCTION__);
+			ret = 0;
+			goto cleanup;
 		}
 	}
+	else	/* for unicode drivers require ANSI behavior */
 #endif /* UNICODE_SUPPORT */
+	{
+		if (!SQL_SUCCEEDED(CC_send_client_encoding(self, self->locale_encoding)))
+		{
+			ret = 0;
+			goto cleanup;
+		}
+	}
 
 	if (self->server_isolation != self->isolation)
 		if (!CC_set_transact(self, self->isolation))
@@ -1047,9 +1128,9 @@ CC_connect(ConnectionClass *self, char *salt_para)
 		CC_clear_error(self);		/* clear any initial command errors */
 	self->status = CONN_CONNECTED;
 	if (CC_is_in_unicode_driver(self)
-	    && 0 < ci->bde_environment)
+	    && (CC_is_in_ansi_app(self) || 0 < ci->bde_environment))
 		self->unicode |= CONN_DISALLOW_WCHAR;
-mylog("conn->unicode=%d\n", self->unicode);
+mylog("conn->unicode=%d Client Encoding='%s' (Code %d)\n", self->unicode, self->original_client_encoding, self->ccsc);
 	ret = 1;
 
 cleanup:
